@@ -6,7 +6,7 @@ use Module::Build;
 use ExtUtils::CppGuess ();
 
 our @ISA = qw(Module::Build);
-our $VERSION = '0.10';
+our $VERSION = '0.13';
 
 # TODO
 # - configurable set of xsp and xspt files (and XS typemaps?)
@@ -121,6 +121,10 @@ sub ACTION_code {
   $self->depends_on('generate_main_xs');
 
   my $files = {};
+  foreach my $file (@{$self->cpp_source_files}) {
+    $files->{$file} = undef;
+  }
+
   foreach my $ext (qw(c cc cxx cpp C)) {
     foreach my $dir (@{$self->cpp_source_dirs||[]}) {
       my $this = $self->_find_file_by_type($ext, $dir);
@@ -141,6 +145,14 @@ sub ACTION_code {
   return $self->SUPER::ACTION_code(@_);
 }
 
+# I guess I should use a module here.
+sub _naive_shell_escape {
+  my $s = shift;
+  $s =~ s/\\/\\\\/g;
+  $s =~ s/"/\\"/g;
+  $s
+}
+
 sub ACTION_generate_main_xs {
   my $self = shift;
 
@@ -208,11 +220,11 @@ MODULE = $module_name	PACKAGE = $module_name
 HERE
 
   my $typemap_args = '';
-  $typemap_args .= '-t ' . Cwd::abs_path($_) . ' ' foreach keys %$xspt_files;
+  $typemap_args .= '-t "' . _naive_shell_escape(Cwd::abs_path($_)) . '" ' foreach keys %$xspt_files;
 
   foreach my $xsp_file (keys %$xsp_files) {
-    my $full_path_file = Cwd::abs_path($xsp_file);
-    my $cmd = "INCLUDE_COMMAND: \$^X -MExtUtils::XSpp::Cmd -e xspp -- $typemap_args $full_path_file\n\n";
+    my $full_path_file = _naive_shell_escape( Cwd::abs_path($xsp_file) );
+    my $cmd = qq{INCLUDE_COMMAND: \$^X -MExtUtils::XSpp::Cmd -e xspp -- $typemap_args "$full_path_file"\n\n};
     $xs_code .= $cmd;
   }
 
@@ -437,6 +449,7 @@ sub _infer_xs_spec {
   return \%spec;
 }
 
+__PACKAGE__->add_property( 'cpp_source_files'      => [] );
 __PACKAGE__->add_property( 'cpp_source_dirs'       => ['src'] );
 __PACKAGE__->add_property( 'build_dir'             => 'buildtmp' );
 __PACKAGE__->add_property( 'extra_xs_dirs'         => [".", grep { -d $_ and /^xsp?$/i } glob("*")] );
@@ -580,7 +593,7 @@ You may use multiple F<.map> files if the entries do not
 collide. They will be merged at build time into a complete F<typemap> file
 in the temporary build directory.
 
-The C<extra_typemap_modules> option is the prefered way to do XS typemapping.
+The C<extra_typemap_modules> option is the preferred way to do XS typemapping.
 It works like any other C<Module::Build> argument that declares dependencies
 except that it loads the listed modules at build time and includes their
 typemaps into the build.
@@ -714,6 +727,8 @@ these magic two lines to your main module:
   require XSLoader;
   XSLoader::load('My::Module', $VERSION);
 
+=back
+
 =head1 SEE ALSO
 
 L<Module::Build> upon which this module is based.
@@ -745,9 +760,11 @@ Mattia Barbon
 
 Shmuel Fomberg
 
+Florian Schlichting
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2010, 2011 Steffen Mueller.
+Copyright 2010, 2011, 2012, 2013 Steffen Mueller.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
